#!/usr/bin/python3
# -*- coding: utf-8 -*-
#------------------------------------------------------------------------
# @(#)dewarp.py Ver. 0.0089 (2022/08/08 14:33:09)
#------------------------------------------------------------------------
# インストール方法:
#   sudo apt-get update
#   sudo apt-get -y upgrade
#   sudo apt-get install -y libopencv-dev
#   sudo apt-get install -y libatlas-base-dev
#   pip3 install opencv-python
#   pip3 install numpy --upgrade
#------------------------------------------------------------------------
# /usr/bin/python3 ./dewarp.py            ---- 全ての bmp ファイルを変換する
# /usr/bin/python3 ./dewarp.py ファイル名 ---- 指定したファイルを変換する
#------------------------------------------------------------------------
import cv2
import os,sys
import numpy as np
import math
import time,datetime
import re
import glob

#------------------------------------------------------------------------
DEBUG=True
#DEBUG=False

#DEBUG2=True
DEBUG2=False

ARC_LENGTH=12

CIRCLE_TXT='circle.txt'
#------------------------------------------------------------------------

#------------------------------------------------------------------------
# 魚眼画像を展開する
#------------------------------------------------------------------------
def dewarp(img):
    center = tuple(np.array([img.shape[1] / 2, img.shape[0] / 2]))
    x,y = center
    cx,cy = int(x), int(y)

    height,width = img.shape[:2]
    size = tuple(np.array([width, height]))

    if (DEBUG == True):
        print("dewarp:center(x,y),width,height=",cx,cy,width,height)
 
    scale = 1.0

    w = ARC_LENGTH
    x = int(cx - w/2)
    y = int(cy)
    h = int(height/2)

    if (DEBUG == True):
        print("dewarp:w,x,y,h=", w,x,y,h)
 
    end_angle = -5 #-20
    for angle in range(360, end_angle, -1):
        rotation_matrix = cv2.getRotationMatrix2D(center, angle, scale)
        img_rot = cv2.warpAffine(img, rotation_matrix, size, flags=cv2.INTER_CUBIC)
        clipped = img_rot[y:y+h, x:x+w : ]
        if (angle == 360):
            img_dewarp = clipped
        else:
            img_dewarp = cv2.hconcat([img_dewarp, clipped])

    return img_dewarp

#------------------------------------------------------------------------
# ファイルに出力する
#------------------------------------------------------------------------
def write_image(img, fname):
    src_fname  = "x_"+fname   #### 入力ファイルに x_ を付けて出力する
    print(" write:", src_fname)
    cv2.imwrite(src_fname, img)
    
#------------------------------------------------------------------------
# 横方向に平均値と最大、最小を求める
# 二値画像から検出するので、thresh は使用していない
#
# 入力
#   img: DEBUG2==True の時、元画像に情報を付加する為に使用
#   binary: 二値画像
#   width : 横幅
#   height: 高さ
#   thresh: 未使用( 二値画像を使用するため)
#   gap   : 処理間隔
# 戻値
#   線分の平均値,線分の最小値,線分の最大値
#------------------------------------------------------------------------
def horizontal(img, binary, width, height, thresh, gap):

    ave = np.array([])  #### 中央値を格納する
    len = np.array([])  #### 長さを格納する
    for y in range(0,height,gap):
        a, b = width, 0
        for x in range(0, width):
            pixel = binary[y,x]
            #print("pixel(",y,x,")=",pixel)
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                a = x
                break
        for x in range(width-1, -1, -1):
            pixel = binary[y,x]
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                b = x
                break
        #print("y=", y, " a=", a, " b=", b)
        if (a < b):
            if (DEBUG2): cv2.line(img, (a,y), (b,y), (0,255,0), 5)
            ave = np.append(ave, a+int((b-a)/2))
            len = np.append(len, int((b-a)))

    return  int(np.average(ave)), int(np.min(len)), int(np.max(len))

#------------------------------------------------------------------------
# 縦方向に平均値と最大、最小を求める
#
# 入力
#   img: DEBUG2==True の時、元画像に情報を付加する為に使用
#   binary: 二値画像
#   width : 横幅
#   height: 高さ
#   thresh: 未使用( 二値画像を使用するため)
#   gap   : 処理間隔
# 戻値
#   線分の平均値,線分の最小値,線分の最大値
#------------------------------------------------------------------------
def virtical(img, binary, width, height, thresh, gap):

    ave = np.array([])  #### 中央値を格納する
    len = np.array([])  #### 長さを格納する
    for x in range(0,width,gap):
        a, b = height, 0
        for y in range(0, height):
            pixel = binary[y,x]
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                a = y
                break
        for y in range(height-1, -1, -1):
            pixel = binary[y,x]
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                b = y
                break
        #print("x=", x, " a=", a, " b=", b)
        if (a < b):
            if (DEBUG2): cv2.line(img, (x,a), (x,b), (0,0,255), 5)
            ave = np.append(ave, a+int((b-a)/2))
            len = np.append(len, int((b-a)))

    return  int(np.average(ave)), int(np.min(len)), int(np.max(len))

#------------------------------------------------------------------------
# 中心と半径を取得する
#------------------------------------------------------------------------
def GetCircle():

    f = open(CIRCLE_TXT, 'r')
    data = f.read()
    f.close()

    n = re.findall(r"\d+", data)
    cx = n[0]
    cy = n[1]
    r  = n[2]

    #print("cx=", cx, "cy=", cy, "r=", r)
    return  int(cx),int(cy),int(r)

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
def main(filename):
    img = cv2.imread(filename)
    width  = img.shape[1]
    height = img.shape[0]
    if (DEBUG == True):
        print("width,height=", img.shape[1], img.shape[0])

    #### 入力画像を二値画像に変換する
    gray   = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    ret,binary = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)

    cx, cy, r = GetCircle()

    #### 検出領域を取り出す
    roi = img[cy-r:cy+r, cx-r:cx+r]

    if (DEBUG == True):
        print("cx,cy,r=", cx, cy, r)
        print("cy-r:cy+r,cx-r:cx+r=", cy-r,cy+r, cx-r,cx+r)
        print("dst[",cy-r,":", cy+r, ",", cx-r,":", cx+r,"]")

        img = cv2.line(img, (cx,cy), (cx,cy-r), (  0,255,  0), thickness=5)
        img = cv2.line(img, (cx,cy), (cx,cy+r), (255,  0,255), thickness=5)
        img = cv2.circle(img, (int(cx), int(cy)), int(r), (0, 0, 255), thickness=5)
        img = cv2.circle(img, (int(cx), int(cy)), int(20), (255, 0, 0), thickness=20)

    if (DEBUG == True):
        st = time.time()

    #### 補正する
    img_dewarp = dewarp(roi)

    if (DEBUG == True):
        et = time.time()
        print("dewarp time:", et-st)

    #### 補正した画像を書き出す
    write_image(img_dewarp, filename)

    if (DEBUG == True):
        dst_1 = cv2.resize(img,        dsize=None, fx=0.25, fy=0.25)
        dst_2 = cv2.resize(gray,       dsize=None, fx=0.25, fy=0.25)
        dst_3 = cv2.resize(binary,     dsize=None, fx=0.25, fy=0.25)
        dst_4 = cv2.resize(roi,        dsize=None, fx=0.25, fy=0.25)
        dst_5 = cv2.resize(img_dewarp, dsize=None, fx=0.25, fy=0.25)

        cv2.imshow("src",    dst_1)
        cv2.imshow("gray",   dst_2)
        cv2.imshow("bin",    dst_3)
        cv2.imshow("roi",    dst_4)
        cv2.imshow("dewarp", dst_5)

        ############
        while (True):
            key = cv2.waitKey(0) & 0xFF
            if (key == 27 or key == ord('q')):
                break
   
        cv2.destroyAllWindows()

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
if __name__ == '__main__':
    if (os.path.exists(CIRCLE_TXT) == False):
        print("#################################")
        print("dewarp: can't find", CIRCLE_TXT)
        print("#################################")
        quit()
   
    args = sys.argv
    if (len(args) >= 2):
        file = args[1]
        is_file = os.path.isfile(file)
        if (is_file):
            xfile = 'x_' + file
            is_file = os.path.isfile(xfile)
            if (is_file):
                print("  skip:", file)
            else:
                DEBUG=True
                main(file)
    else:
        files = glob.glob("[0-9]*.bmp")
        DEBUG=False

        for file in files:
            print("dewarp:", file)
            xfile = 'x_' + file
            is_file = os.path.isfile(xfile)
            if (is_file):
                print("  skip:", file)
            else:
                main(file)

    print("#################################")
    print("dewarp: complete")
    print("#################################")

