#!/usr/bin/python3
# -*- coding: utf-8 -*-
#------------------------------------------------------------------------
# @(#)get_circle.py Ver. 0.0087 (2022/08/08 14:33:09)
#------------------------------------------------------------------------
# インストール方法:
#   sudo apt-get update
#   sudo apt-get -y upgrade
#   sudo apt-get install -y libopencv-dev
#   sudo apt-get install -y libatlas-base-dev
#   pip3 install opencv-python
#   pip3 install numpy --upgrade
#------------------------------------------------------------------------
import cv2
import os,sys
import numpy as np
import math
import time,datetime

#------------------------------------------------------------------------
#DEBUG=True
DEBUG=False

#DEBUG2=True
DEBUG2=False

#------------------------------------------------------------------------
# 横方向に平均値と最大、最小を求める
# 二値画像から検出するので、thresh は使用していない
#
# 入力
#   img: DEBUG2==True の時、元画像に情報を付加する為に使用
#   binary: 二値画像
#   width : 横幅
#   height: 高さ
#   thresh: 未使用( 二値画像を使用するため)
#   gap   : 処理間隔
# 戻値
#   線分の平均値,線分の最小値,線分の最大値
#------------------------------------------------------------------------
def horizontal(img, binary, width, height, thresh, gap):

    ave = np.array([])  #### 中央値を格納する
    len = np.array([])  #### 長さを格納する
    for y in range(0,height,gap):
        a, b = width, 0
        for x in range(0, width):
            pixel = binary[y,x]
            #print("get_circle:pixel(",y,x,")=",pixel)
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                a = x
                break
        for x in range(width-1, -1, -1):
            pixel = binary[y,x]
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                b = x
                break
        #print("get_circle:y=", y, " a=", a, " b=", b)
        if (a < b):
            if (DEBUG2): cv2.line(img, (a,y), (b,y), (0,255,0), 5)
            ave = np.append(ave, a+int((b-a)/2))
            len = np.append(len, int((b-a)))

    return  int(np.average(ave)), int(np.min(len)), int(np.max(len))

#------------------------------------------------------------------------
# 縦方向に平均値と最大、最小を求める
#
# 入力
#   img: DEBUG2==True の時、元画像に情報を付加する為に使用
#   binary: 二値画像
#   width : 横幅
#   height: 高さ
#   thresh: 未使用( 二値画像を使用するため)
#   gap   : 処理間隔
# 戻値
#   線分の平均値,線分の最小値,線分の最大値
#------------------------------------------------------------------------
def virtical(img, binary, width, height, thresh, gap):

    ave = np.array([])  #### 中央値を格納する
    len = np.array([])  #### 長さを格納する
    for x in range(0,width,gap):
        a, b = height, 0
        for y in range(0, height):
            pixel = binary[y,x]
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                a = y
                break
        for y in range(height-1, -1, -1):
            pixel = binary[y,x]
            #if (pixel > thresh):
            if (pixel != 0):
                if (DEBUG2): cv2.circle(img, (x, y), int(10), (255,0,0), 10)
                b = y
                break
        #print("get_circle:x=", x, " a=", a, " b=", b)
        if (a < b):
            if (DEBUG2): cv2.line(img, (x,a), (x,b), (0,0,255), 5)
            ave = np.append(ave, a+int((b-a)/2))
            len = np.append(len, int((b-a)))

    return  int(np.average(ave)), int(np.min(len)), int(np.max(len))

#------------------------------------------------------------------------
# 中心と半径を求める
#
# 入力
#   img: DEBUG2==True の時、元画像に情報を付加する為に使用
#   binary: 二値画像
#   width : 横幅
#   height: 高さ
#   thresh: 未使用. 二値画像を使用するため
#   gap   : 処理間隔
# 戻値
#   cx    : 中心点x
#   cy    : 中心点y
#   r     : 半径
#------------------------------------------------------------------------
def GetCenter(img, binary, width, height, thresh, gap):
    cx,xmin,xmax = horizontal(img, binary, width, height, thresh, gap)
    cy,ymin,ymax = virtical  (img, binary, width, height, thresh, gap)
    xr = xmax // 2
    yr = ymax // 2
    if (xr > yr):
        r = xr
    else:
        r = yr

    r += 100  #### 半径をちょっと多めにする
    if (r + r > height):   ### 半径が画角をはみ出た場合の処理
        if (cy < height - cy):
            r = cy
        else:
            r = height - cy

    return  cx, cy, r

##################################
#### 処理速度が遅いため、却下
##################################
####def GetCenter(img):
####    gray = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
####
####    ret, binary = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)
####    contours, hierarchy = cv2.findContours(binary, cv2.RETR_EXTERNAL, cv2.CHAIN_APPROX_SIMPLE)
####    max_contour = max(contours, key=lambda x: cv2.contourArea(x))
####    (cx, cy), r = cv2.minEnclosingCircle(max_contour)
####
####    return  cx, cy, r


#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
def main(filename):
    img = cv2.imread(filename)
    width  = img.shape[1]
    height = img.shape[0]
    if (DEBUG == True):
        print("get_circle:width,height=", img.shape[1], img.shape[0])

    #### 入力画像を二値画像に変換する
    gray   = cv2.cvtColor(img, cv2.COLOR_BGR2GRAY)
    ret,binary = cv2.threshold(gray, 0, 255, cv2.THRESH_BINARY + cv2.THRESH_OTSU)

    #### 中心点と半径を求める
    thresh = 20   #### 二値画像から検出するので,thresh は使用していない
    gap = 50      #### 小さくするほど精度はあがるが,時間はかかる
    cx, cy, r = GetCenter(img, binary, width, height, thresh, gap)

    #### 検出領域を取り出す
    roi = img[cy-r:cy+r, cx-r:cx+r]

    if (DEBUG == True):
        print("get_circle:cx,cy,r=", cx, cy, r)
        print("get_circle:cy-r:cy+r,cx-r:cx+r=", cy-r,cy+r, cx-r,cx+r)
        print("get_circle:dst[",cy-r,":", cy+r, ",", cx-r,":", cx+r,"]")

        img = cv2.line(img, (cx,cy), (cx,cy-r), (  0,255,  0), thickness=5)
        img = cv2.line(img, (cx,cy), (cx,cy+r), (255,  0,255), thickness=5)
        img = cv2.circle(img, (int(cx), int(cy)), int(r), (0, 0, 255), thickness=5)
        img = cv2.circle(img, (int(cx), int(cy)), int(20), (255, 0, 0), thickness=20)

    if (DEBUG == True):
        st = time.time()

    if (DEBUG == True):
        dst_1 = cv2.resize(img,        dsize=None, fx=0.25, fy=0.25)
        dst_2 = cv2.resize(gray,       dsize=None, fx=0.25, fy=0.25)
        dst_3 = cv2.resize(binary,     dsize=None, fx=0.25, fy=0.25)
        dst_4 = cv2.resize(roi,        dsize=None, fx=0.25, fy=0.25)

        cv2.imshow("src",    dst_1)
        cv2.imshow("gray",   dst_2)
        cv2.imshow("bin",    dst_3)
        cv2.imshow("roi",    dst_4)

        ############
        while (True):
            key = cv2.waitKey(0) & 0xFF
            if (key == 27 or key == ord('q')):
                break
   
        cv2.destroyAllWindows()

    with open('circle.txt', 'w') as f:
       circle = "{:4d} {:4d} {:4d}".format(cx, cy, r)
       f.write(circle)
       f.close()

#------------------------------------------------------------------------
#
#------------------------------------------------------------------------
if __name__ == '__main__':
    args = sys.argv
    if (len(args) >= 2):
        is_file = os.path.isfile(args[1])
        if (is_file):
            main(args[1])
            print("#################################")
            print("get_circle: complete")
            print("#################################")
        else:
            print("#################################")
            print("get_circle.py:", args[1], "not found.")
            print("#################################")
    else:
        print(args[0], "filename")
