#!/usr/bin/python3
#-----------------------------------------------------------------------
# @(#)gui.py Ver. 0.0011 (2022/08/08 14:33:09)
#-----------------------------------------------------------------------
import os, signal, sys, psutil, datetime, time
import tkinter
from tkinter import *
from tkinter import ttk
import subprocess
from subprocess import Popen

#-----------------------------------------------------------------------
# 初期値設定
#-----------------------------------------------------------------------
geometry    = '+5+40'
title       = 'endoscape'

#-----------------------------------------------------------------------

STATE_NULL=0
STATE_VID=1

TYPE_PREVIEW=0
TYPE_GET_CIRCLE=1
TYPE_DEWARP=2
TYPE_STILL=3
TYPE_CONV_ALL=4

pid = 0
state=STATE_NULL

#JP=False
JP=True

if (JP == False):
    BN_PREVIEW='Preview'
    BN_GET_CIRCLE='Get Circle'
    BN_DEWARP='Dewap(for check)'
    BN_STILL='STILL'
    BN_CONV_ALL='Dewarp ALL'
    BN_QUIT='Quit'
else:
    BN_PREVIEW='プレビュー'
    BN_GET_CIRCLE='円を求める'
    BN_DEWARP='補正(確認用)'
    BN_STILL='撮影'
    BN_CONV_ALL='一括補正変換'
    BN_QUIT='終了'

#-----------------------------------------------------------------------
# ビデオ再生を止める
#-----------------------------------------------------------------------
def stop():
    global pid

    if (pid != 0):
        print("stop: pid=", pid)
        kill(pid)

#-----------------------------------------------------------------------
# プログラムを終了する
#-----------------------------------------------------------------------
def quit():
    global win
    global pid

    if (pid != 0):
        os.killpg(os.getpgid(pid), signal.SIGKILL)
    win.quit()

#-----------------------------------------------------------------------
# 指定されたプロセスを kill する
#-----------------------------------------------------------------------
def kill(pid):
    if (psutil.pid_exists(pid)):
        os.kill(pid, signal.SIGKILL)
    if (psutil.pid_exists(pid+1)):
        os.kill(pid+1, signal.SIGKILL)

#-----------------------------------------------------------------------
# ファイル名を作成する
#
#   IMX258_202207111258_s30_g32_ev0_wb0_sh1.h264
#-----------------------------------------------------------------------
def MakeFileName(ext):  # ext=拡張子( .bmp .jpg ...)
    dt_now = datetime.datetime.now()
    dt_str = dt_now.strftime('%Y%m%d_%H%M%S')

    fname = dt_str + ext

    return  fname

#-----------------------------------------------------------------------
# 実行する
#-----------------------------------------------------------------------
def execute(type, fname):
    global proc, pid, state

    if (pid != 0):
        kill(pid)

    if (type == TYPE_PREVIEW):
        command = 'libcamera-hello -t 0 --tuning-file=/home/pi/libcamera/src/ipa/raspberrypi/data/imx258_noir.json'
    elif (type == TYPE_GET_CIRCLE):
        command = '/usr/bin/bash ./get_circle.sh'
    elif (type == TYPE_DEWARP):
        command = '/usr/bin/bash ./cap_dewarp.sh'
    elif (type == TYPE_STILL):
        command = 'libcamera-still --tuning-file=/home/pi/libcamera/src/ipa/raspberrypi/data/imx258_noir.json -t 1000 -o ' + fname
    elif (type == TYPE_CONV_ALL):
        command = '/usr/bin/python3 ./dewarp.py'

    print(command)
    proc = Popen(command, shell=True)
    pid = proc.pid
    print("execute:pid=", pid)

#-----------------------------------------------------------------------
# 設定データを表示し、ファイル名が必要な場合は作成する
#-----------------------------------------------------------------------
def do_exec(type):

    fname = ''
    if (type == TYPE_PREVIEW):       #Preview
        print("Preview")
    elif (type == TYPE_GET_CIRCLE):  #GetCircle
        fname = MakeFileName(".bmp")
        print("GET CIRCLE:", fname)
    elif (type == TYPE_DEWARP):      #DEWARP
        fname = MakeFileName(".bmp")
        print("DEWARP:", fname)
    elif (type == TYPE_STILL):       #STILL
        fname = MakeFileName(".bmp")
        print("STILL:", fname)
    elif (type == TYPE_CONV_ALL):    #CONV_ALL
        print("CONV_ALL:")

    execute(type, fname)

#-----------------------------------------------------------------------
# ウインドウを表示する
#-----------------------------------------------------------------------
def window():
    global win

    win = Tk()
    win.geometry(geometry)
    win.title(title)

    row_val = 0
    ##########################################################
    # PREVIEW/GetCircle/DEWARP
    ##########################################################
    Button(win, font=("",14),text=BN_PREVIEW, bg="yellow",fg="black", command=lambda : do_exec(TYPE_PREVIEW)).grid(column=0,row=row_val,columnspan=2,sticky="NSEW")

    row_val += 1
    ##########################################################
    # 空行
    ##########################################################
    Label(win, text="", font=("",4)).grid(column=0,row=row_val)

    row_val += 1
    ##########################################################
    # GetCircle / DEWARP
    ##########################################################
    Button(win, font=("",14),text=BN_GET_CIRCLE,bg="blue",fg="white",  command=lambda : do_exec(TYPE_GET_CIRCLE)).grid(column=0,row=row_val,columnspan=2,sticky="NSEW")
    row_val += 1
    Button(win, font=("",14),text=BN_DEWARP,bg="purple",fg="white",      command=lambda : do_exec(TYPE_DEWARP)).grid(column=0,row=row_val,columnspan=2,sticky="NSEW")

    row_val += 1
    ##########################################################
    # 空行
    ##########################################################
    Label(win, text="", font=("",4)).grid(column=0,row=row_val)

    row_val += 1
    ##########################################################
    # STILL / CONV_ALL
    ##########################################################
    Button(win, font=("",14),text=BN_STILL,bg="blue",fg="white",       command=lambda : do_exec(TYPE_STILL)).grid(column=0,row=row_val,columnspan=2,sticky="NSEW")
    row_val += 1

    Button(win, font=("",14),text=BN_CONV_ALL,bg="blue",fg="white",     command=lambda : do_exec(TYPE_CONV_ALL)).grid(column=0,row=row_val,columnspan=2,sticky="NSEW")

    row_val += 1
    ##########################################################
    # 空行
    ##########################################################
    Label(win, text="", font=("",4)).grid(column=0,row=row_val)

    row_val += 1
    ##########################################################
    # STOP/Quit
    ##########################################################
    Button(win, font=("",14),text=BN_QUIT,bg="black",fg="white", command=quit).grid(column=0,row=row_val,columnspan=2,sticky="NSEW")

    win.mainloop()

#-----------------------------------------------------------------------
# main
#-----------------------------------------------------------------------
def main():
    global pid, state
    pid = 0
    state = STATE_NULL
    window()

#-----------------------------------------------------------------------
#
#-----------------------------------------------------------------------
if __name__ == "__main__":
    main()
